/*
  Windows Sockets FTP Application

  Written by: John A. Junod            Internet: <junodj@gordon-emh2.army.mil>
              267 Hillwood Street                <zj8549@trotter.usma.edu>
              Martinez, GA 30907     Compuserve: 72321,366 

  This program executable and all source code is released into the public
  domain.

  MODULE: WS_IP.C

  Based on code published in UNIX Network Programming by W. Richard Stevens

  These are common routines that will be needed by many different programs
  and should not be modified for a specific instance.
*/
#include "ws_glob.h"
#include "ws_ftp.h"
#include <stdlib.h>

SOCKET connectTCP(char *host,char *service)
{
  return connectsock(host,service,"tcp");
}

SOCKET connectUDP(char *host,char *service)
{
  return connectsock(host,service,"udp");
}

SOCKET connectsock(char *host,char *service,char *protocol)
{
  struct hostent *phe;    // pointer to host entry
  struct servent *pse;    // pointer to service entry
  struct protoent *ppe;   // pointer to protocol entry
  SOCKET s;               // socket
  int type;               // socket type
  char szMsgBuf[128];     // general purpose message buffer

  // initialize socket address structure
  bzero((char *)&sin,sizeof(sin));
  sin.sin_family=AF_INET;
  // get service port number from name
  if(pse=getservbyname(service,protocol))
    sin.sin_port=pse->s_port;
  else if((sin.sin_port=htons((u_short)atoi(service)))==0) {
    DoPrintf("can't get \"%s\" service entry",service);
    return INVALID_SOCKET;
  }
  // map host to ip, allow ip
  if(phe=gethostbyname(host))
    bcopy(phe->h_addr,(char *)&sin.sin_addr,phe->h_length);
  else if((sin.sin_addr.s_addr=inet_addr(host))==INADDR_NONE) {
    DoPrintf("can't get \"%s\" host entry",host);
    return INVALID_SOCKET;
  }
  // map protocol name to protocol number
  if((ppe=getprotobyname(protocol))==0) {
    DoPrintf("can't get \"%s\" protocol entry",protocol);
    return INVALID_SOCKET;
  }
  // use protocol to choose socket type
  if(strcmp(protocol,"udp")==0)
    type=SOCK_DGRAM;
  else
    type=SOCK_STREAM;
  // allocate a socket
  s=socket(AF_INET,type,ppe->p_proto);
  if(s==INVALID_SOCKET) {
    DoAddLine("can't create socket");
    ReportWSError(WSAGetLastError());
    return INVALID_SOCKET;
  }

  bcopy((LPSTR)&dsin,(LPSTR)&sin,sizeof(sin));
  dsin.sin_port=htons(20);

  // connect the socket
  if(connect(s,(struct sockaddr *)&sin,sizeof(sin))==SOCKET_ERROR) {
    DoPrintf("Can't connect to %s.%s",host,service);
    ReportWSError(WSAGetLastError());
    return INVALID_SOCKET;
  }

  DoPrintf("Connected to %s port %u",
    inet_ntoa(sin.sin_addr),ntohs(sin.sin_port));

  return s;
}

int sendstr(SOCKET sockfd,LPSTR ptr,int nbytes)
{
  int nleft,nwritten;
  nleft=nbytes;
  while(nleft > 0) {                   // while we haven't written enough
    nwritten=send(sockfd,ptr,nleft,0); // write what we can
    if(nwritten <= 0) {
      ReportWSError(WSAGetLastError());
      return(nwritten);  // error occured
    }
    nleft -= nwritten;                   // count what we wrote
    ptr   += nwritten;                   // adjust buffer pointer
  }
  return(nbytes - nleft);              // return count of bytes written
}

